/*
 * Simpe webclient
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/types.h> 
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <fcntl.h>
#include <time.h>


#define BUFSIZE   4096
#define GETSTRING "GET %s HTTP/1.0\n\n"		/* Magic http header */

void error();

int main(int argc, char *argv[])
{

	char *url;
	char *hostname;
	int port;
	int fd;
	void sendRequest();
	void getReply();
	
	if (argc != 4) {
		fprintf(stderr,"Usage: %s hostname portnumber url\n", argv[0]);
		exit(-1);
	}

	hostname = argv[1];
	port = atoi(argv[2]);
	url = argv[3];

	fd = connectToServer(hostname, port);
	sendRequest(fd, url);
	getReply(fd);
	close(fd);
}



int
connectToServer(char *hostname, int port)
{
	struct sockaddr_in serv_addr;
	struct hostent *server;
	int sfd;
	
	sfd = socket(AF_INET, SOCK_STREAM, 0);
	if (sfd < 0) 
		error("ERROR opening socket");
	
	server = gethostbyname(hostname);
	if (server == NULL) {
		fprintf(stderr,"ERROR, no such host\n");
		exit(0);
	}
	
	bzero((char *) &serv_addr, sizeof(serv_addr));
	serv_addr.sin_family = AF_INET;
	bcopy((char *)server->h_addr, 
	      (char *)&serv_addr.sin_addr.s_addr,
	      server->h_length);
	
	serv_addr.sin_port = htons(port);
	if (connect(sfd, (struct sockaddr*)&serv_addr,sizeof(serv_addr)) < 0) 
		error("ERROR connecting");
	return sfd;
}


void
sendBuf(int tofd, char *buf, int sz)
{
	int cc;
	int n = sz;

	/* Pump through buffer writting as much as we can until empty. */
	while (n > 0) {
		cc = write(tofd, &buf[sz - n], n);
		if (cc < 0)
			error("sendBuf");
		else
			n -= cc;
	}
}


void
sendRequest(int fd, char *url)
{
	char buf[BUFSIZE];
	sprintf(buf, GETSTRING, url);
	sendBuf(fd, buf, strlen(buf) + 1);  /* include null terminator */
}


void
getReply(int fd)
{
	char buf[BUFSIZE];
	int inHeader = 1;

	for (;;) {
		int cc = read(fd, buf, sizeof(buf)-1);
		if (cc < 0)
			error("getReply:read");
		if (cc == 0)
			break;
		
		/* ensure we have a null terminator */		
		buf[cc+1] = 0;
		if (inHeader && cc > 4) {
			/* assume header arrives in first read */
			int i;
			for (i = 0; i < cc-4; i++) {
				if (buf[i] == '\r' && buf[i+1] == '\n' &&
				    buf[i+2] == '\r' && buf[i+3] == '\n') {
					char *cp = &buf[i+4];
					printf("%s", cp);
					inHeader = 0;
				}
			}

		} else 	
			printf("%s", buf);
	}
}

	
void error(char *msg)
{
	perror(msg);
	exit(1);
}

