/*
 * Simple webserver
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/types.h> 
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <fcntl.h>
#include <time.h>


#define BUFSIZE   4096
#define GETTOKEN "GET "		/* Magic http token */

void error();


int main(int argc, char *argv[])
{

	int port;
	char *servicedir;
	int listenfd;
	int callerfd;
	void serviceRequest();
		
	if (argc != 3) {
		fprintf(stderr,"Usage: %s portnumber servicedir\n", argv[0]);
		exit(-1);
	}

	port = atoi(argv[1]);
	servicedir = argv[2];

	listenfd = createSocket(port);

	for (;;) {

		callerfd = awaitRequest(listenfd);
		serviceRequest(servicedir, callerfd);
	}
}



int
createSocket(int port)
{
	struct sockaddr_in serv_addr;
	int sockfd;
	sockfd = socket(AF_INET, SOCK_STREAM, 0);
     
	bzero((char *) &serv_addr, sizeof(serv_addr));
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr.s_addr = INADDR_ANY;
	serv_addr.sin_port = htons(port);
	if (bind(sockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0) 
		error("ERROR on binding");
	listen(sockfd,5);
	return sockfd;
}


char *timenow()
{
	time_t now;
	char *cp;
	now = time( (time_t*)0);
	cp = ctime(&now);
	cp[strlen(cp)-1] = 0;  /* kill newline */
	return cp;
}

	
int
awaitRequest(int listenfd)
{
	struct sockaddr_in cliaddr;
	socklen_t clilen;
	clilen = sizeof(cliaddr);
	int callerfd;

	char hostname[NI_MAXHOST];
	char portnumber[16];

	callerfd = accept(listenfd,  (struct sockaddr *) &cliaddr,  &clilen);
	if (callerfd < 0) 
		error("awaitRequest: ERROR on accept");



	if (getnameinfo( (struct sockaddr*)&cliaddr,
			 clilen,
			 hostname, sizeof(hostname),
			 portnumber, sizeof(portnumber),
			 NI_NUMERICSERV) < 0)
		error("awaitRequest: getnameinfo");

	
	fprintf(stderr, "New request: %s %s %s\n", timenow(), hostname, portnumber);
		
 	return callerfd;
}




void
serviceRequest(char *servicedir, int callerfd)
{


	char *url;
	int urlfd;
	char buf[BUFSIZE];
	char *getHeader();
	char *parseHeader();
	void sendFile();
	void sendBuf();

	url = parseHeader(getHeader(callerfd, buf, sizeof(buf)));
	if (!url || strlen(url) + strlen(servicedir) > sizeof(buf)) {
		error("serviceRequest: bad URL");
	}

	sprintf(buf, "%s%s", servicedir, url);
	
	urlfd = open(buf, O_RDONLY);
	
	fprintf(stderr,"REQUESTED  %s\n", buf);
	
	if (urlfd < 0) {
		char errbuf[BUFSIZ];
		fprintf(stderr, "serviceRequest: open\n");
		sprintf(errbuf, "400 %s FILE NOT FOUND\n", buf);
		sendBuf(callerfd, errbuf, strlen(errbuf)+1);
	} else
		sendFile(urlfd, callerfd);
	
	close(urlfd);
	shutdown(callerfd, SHUT_RDWR);
	close(callerfd);
}


char *
getHeader(int callerfd, char* buf, int bufsz)
{
	int cc = read(callerfd, buf, bufsz);
	if (cc <= 0) {
		fprintf(stderr, "getHeader: Bad read");
		return NULL;
	}
	buf[cc] = 0;  /* ensure null terminated */
	fprintf(stderr,"%s\n", buf);
	return buf;

}


  



void
sendBuf(int tofd, char *buf, int sz)
{
	int cc;
	int n = sz;

	/* Pump through buffer writting as much as we can until empty. */
	while (n > 0) {
		cc = write(tofd, &buf[sz - n], n);
		if (cc < 0)
			error("sendBuf");
		n -= cc;
	}
}

	
			      
void
sendFile(int fromfd, int tofd)
{
	int cc;
	char buf[BUFSIZE];
	
	for (;;) {
		cc = read(fromfd, buf, sizeof(buf));
		if (cc < 0)
			error("sendFile: readurl");
		if (cc == 0) 
			break;
		sendBuf(tofd, buf, cc);
	}
}

char *
parseHeader(char *buf)
{
	char *url;
	int i;

	if (strncmp(buf, GETTOKEN, strlen(GETTOKEN)) != 0) {
		fprintf(stderr, "parseHeader: Bad Header:%s\n", buf);
		return NULL;
	}

	url = &buf[strlen(GETTOKEN)];

	/* replace first space with null terminator */
	for (i = strlen(GETTOKEN); i < strlen(buf); i++) {
		if (buf[i] == ' '){
			buf[i] = 0;
			return url;
		}
	}

	fprintf(stderr,"parseHeader: badURL: %s\n", buf);
	return NULL;
}

		  



void error(char *msg)
{
	perror(msg);
	exit(1);
}

